#version 120

#define Godrays
#define Fog
//#define ShadowColor
#define SSEO
//#define SSAO
#define WaterRefraction
//#define WorldTimeAnimation

const int shadowMapResolution = 2048; //[1024 2048 4096]

#define SHADOW_MAP_BIAS 0.9
#define MAX_COLOR_RANGE 32.0
const bool colortex0MipmapEnabled = true;

varying vec3 lightVector;
varying vec3 upVec;
varying vec3 sunVec;
varying vec3 moonVec;
varying float sunVisibility;
varying float moonVisibility;
varying float handRecolor;

varying vec4 texcoord;

uniform sampler2D colortex0;
uniform sampler2D colortex1;
uniform sampler2D colortex2;
uniform sampler2D colortex3;
uniform sampler2D colortex4;
uniform sampler2D colortex5;
uniform sampler2D depthtex0;
uniform sampler2D noisetex;

uniform sampler2DShadow shadowtex0;
#ifdef ShadowColor
uniform sampler2DShadow shadowtex1;
uniform sampler2DShadow shadowcolor;
#endif

uniform mat4 gbufferProjection;
uniform mat4 gbufferProjectionInverse;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferModelView;
uniform mat4 shadowProjection;
uniform mat4 shadowModelView;
uniform float rainStrength;
uniform ivec2 eyeBrightnessSmooth;
uniform int isEyeInWater;
uniform int worldTime;
uniform float frameTimeCounter;
uniform vec3 cameraPosition;
uniform vec3 sunPosition;
uniform float viewWidth;
uniform float viewHeight;
uniform float far;
uniform float near;
uniform float aspectRatio;

float time = float(worldTime);
float tB = max(sin(time/12000*22/7),0.0);
float eBS = eyeBrightnessSmooth.y/240.0;

#ifdef WorldTimeAnimation
float frametime = float(worldTime)/20.0;
#else
float frametime = frameTimeCounter;
#endif

float nolight = 1.0-(clamp((time-12000.0)/500.0,0.0,1.0)-clamp((time-13500.0)/500.0,0.0,1.0) + clamp((time-22500.0)/500.0,0.0,1.0)-clamp((time-23500.0)/500.0,0.0,1.0));

vec3 calcLightColor(vec3 morning, vec3 day, vec3 night, vec3 rain){
vec3 c = mix(night,mix(morning,day,pow(tB,0.75)),sunVisibility);
return mix(c,length(c)*rain,rainStrength);
}

float Blinn_Phong(vec3 ppos, vec3 lvector, vec3 normal, float gloss, float visibility)  {
	vec3 lightDir = vec3(lvector);
	
	vec3 surfaceNormal = normal;
	float cosAngIncidence = dot(surfaceNormal, lightDir);
	cosAngIncidence = clamp(cosAngIncidence, 0.0, 1.0);	
	
	vec3 viewDirection = normalize(-ppos);
	
	vec3 halfAngle = normalize(lightDir + viewDirection);
	float blinnTerm = dot(surfaceNormal, halfAngle);
	
	float normalDotEye = dot(normal, normalize(ppos));
	float fresnel = clamp(pow(1.0 + normalDotEye, 5.0),0.0,1.0);
	fresnel = fresnel*0.85 + 0.15 * (1.0-fresnel);
	float pi = 3.1415927;
	float n =  pow(2.0,gloss);
	return (pow(blinnTerm, n )*((n+8.0)/(8*pi)))*visibility;
}

float waterH(vec3 pos) {
float noise = 0;
noise+= texture2D(noisetex,(pos.xz+vec2(frametime)*0.5-pos.y*0.2)/1024.0* 1.1).r*1.0;
noise+= texture2D(noisetex,(pos.xz-vec2(frametime)*0.5-pos.y*0.2)/1024.0* 1.5).r*0.8;
noise-= texture2D(noisetex,(pos.xz+vec2(frametime)*0.5+pos.y*0.2)/1024.0* 2.5).r*0.6;
noise+= texture2D(noisetex,(pos.xz-vec2(frametime)*0.5-pos.y*0.2)/1024.0* 5.0).r*0.4;
noise-= texture2D(noisetex,(pos.xz+vec2(frametime)*0.5+pos.y*0.2)/1024.0* 8.0).r*0.2;

noise /= pow(max(length(pos),4.0),0.4);
return noise;
}

vec2 getRefract(vec2 coord, vec3 posxz){
float deltaPos = 0.1;
float h0 = waterH(posxz);
float h1 = waterH(posxz + vec3(deltaPos,0.0,0.0));
float h2 = waterH(posxz + vec3(-deltaPos,0.0,0.0));
float h3 = waterH(posxz + vec3(0.0,0.0,deltaPos));
float h4 = waterH(posxz + vec3(0.0,0.0,-deltaPos));

float xDelta = ((h1-h0)+(h0-h2))/deltaPos;
float yDelta = ((h3-h0)+(h0-h4))/deltaPos;

vec2 newcoord = coord + vec2(xDelta,yDelta)*0.01;
float limit = float(newcoord.x < 0.0 || newcoord.x > 1.0 || newcoord.y < 0.0 || newcoord.y > 1.0);

return mix(newcoord,coord,limit);
}

float ld(float depth) {
   return (2.0 * near) / (far + near - depth * (far - near));
}

float dither4x4(vec2 pos)
{
	const int ditherPattern[16] = int[16](
		0, 8, 2, 10,
		12, 4, 14, 6,
		3, 11, 1, 9,
		15, 7, 13, 5);

    vec2 positon = floor(mod(vec2(texcoord.s * viewWidth,texcoord.t * viewHeight), 4.0f));

	int dither = ditherPattern[int(positon.x) + int(positon.y) * 4];

	return float(dither) / 16.0f;
}

#ifdef Godrays
vec2 getGRCoord(vec2 coord, vec2 dcoord, float str){
return coord - dcoord*dither4x4(texcoord.xy)*str;
}
#endif

#include "lib/fragPos.glsl"
#include "lib/shadowPos.glsl"
#include "lib/shadows.glsl"
#include "lib/lightColors.glsl"
#include "lib/torchColor.glsl"
#include "lib/waterColor.glsl"
#include "lib/fogCalc.glsl"
#include "lib/ambientOcclusion.glsl"

void main() {
	
	
	vec3 color = pow(texture2D(colortex0, texcoord.st).rgb,vec3(2.2))*MAX_COLOR_RANGE;
	vec3 aux = texture2D(colortex5, texcoord.st).rgb;
	float ao = 1.0;
	float spec = texture2D(colortex0, texcoord.st).a;
	vec3 gr = vec3(0.0);
	
	vec3 light = calcLightColor(light_m, light_d + vec3(0.0, 0.25, 1.0), light_n, light_r);
	vec3 ambient = calcLightColor(sky_m, sky_d, sky_n, sky_r);
	
	float skymap = pow(aux.r,2.2);
	float matflag = aux.g;
	float torchmap = pow(aux.b,2.2)*0.15 + pow(max(pow(aux.b,2.2)-0.5,0.0)*2.0,5.0);
	float alpha = texture2D(colortex1,texcoord.xy).a;
	
	float translucent = float(matflag > 0.05);
	float water = float(matflag > 0.08 && matflag < 0.12);
	float hand = float(matflag > 0.38 && matflag < 0.42);
	
	if (translucent > 0.9) {
		vec3 colorb = pow(max(texture2D(colortex1, texcoord.st).rgb,vec3(1.0/255.0)),vec3(2.2))/pow(alpha,2.2);
		vec3 normal = texture2D(colortex3, texcoord.st).rgb*2.0-1.0;
		//colorb = vec3(1.0,1.0,1.0)*mix(0.4,0.01,water);
		
		float pixeldepth = texture2D(depthtex0,texcoord.xy).x;
	
		vec4 fragpos = getFragPos(texcoord.xy,pixeldepth);
		vec4 worldpos = getWorldPos(fragpos);
		
		#ifdef WaterRefraction
		if (water > 0.9 && isEyeInWater < 0.9){
			vec2 newcoord = getRefract(texcoord.xy,worldpos.xyz+cameraPosition.xyz);
			color = pow(texture2D(colortex0, newcoord).rgb,vec3(2.2))*MAX_COLOR_RANGE;
		}
		#endif
		
		color *= mix(vec3(1.0),colorb,pow(alpha,1.0/2.2));
		
		if (handRecolor > 0.9 && hand > 0.9){
			float ec = pow(length(colorb),2.8)*4.0;
			colorb = ec*pow(torch_c,vec3(2.2))*0.4+ec*0.35;
			}
		
		
		vec3 finallight = vec3(0);
		vec3 scenelight = vec3(0);
		vec3 inambient = vec3(0.8)*pow(skymap,0.5)*(0.5+0.5*tB)*(1.0-eBS)*(1.0-rainStrength*0.8)*(0.05+0.95*sunVisibility);
		vec3 torchlight = pow(torch_c,vec3(1.2-0.2*pow(aux.b,0.25)))*torchmap*8.0*mix(1.0,pow(1.0/length(colorb),0.5),water);
		
		float absorb = float((water > 0.9 && isEyeInWater < 0.9) || (water < 0.9 && isEyeInWater > 0.9));
		vec3 uw_tint = mix(vec3(1.0),pow(water_c,vec3((1.0-skymap)*2.0))*skymap,absorb);
		
		float NdotL = max(dot(normal, lightVector)*1.05-0.05,0.0);
		float shading = getShadows(shadowtex0, worldpos, NdotL, 0.0);
		shading *= max(NdotL*nolight*(1.0-hand*handRecolor*0.8),0.0);
		
		vec3 shadowcol = vec3(0.0);
		#ifdef ShadowColor
		if (shading < 0.999) shadowcol = getShadowsColor(worldpos, NdotL, 0.0)*getShadows(shadowtex1, worldpos, NdotL, 0.0)*NdotL;
		#endif
		
		if (alpha > 0.98){
			#ifdef SSEO
			ao = sseo(ao,depthtex0,fragpos.xyz);
			#endif
			#ifdef SSAO
			if (hand < 0.9) ao = ssao(depthtex0,normal);
			if (hand > 0.9) ao = sseo(ao,depthtex0,fragpos.xyz);
			#endif
		}
		
		float minlight = 2.0/255.0 * (4.0-3.0*eBS);
		float lightmult = mix(3.0+2.0*shading,1.0,eBS);
		scenelight = mix(((ambient+inambient)*skymap+minlight)*lightmult*ao,light*lightmult,max(vec3(shading),shadowcol)*(1.0-0.9*rainStrength));
		finallight = scenelight*uw_tint+torchlight*ao;
		
		colorb *= finallight;
		
		float desat = clamp(max(pow(shading,0.2),skymap)*sunVisibility*(1-rainStrength*0.8)+aux.b,0.2,1.0);
		vec3 desat_c = mix(vec3(length(light_n))*0.05*sky_r,light_n*2.0,skymap*(1.0-rainStrength));
		colorb = mix(length(colorb)*desat_c,colorb,desat);
		
		color = mix(color,colorb,pow(alpha,2.2));
		
		#ifdef Fog
		if (isEyeInWater < 0.9) color = calcFog(color, light, fragpos.xyz);
		#endif
		
		spec = Blinn_Phong(fragpos.xyz,lightVector,normal,12+moonVisibility,shading*0.25);

	}
	
	#ifdef Godrays
	vec4 tpos = vec4(sunPosition,1.0)*gbufferProjection;
	tpos = vec4(tpos.xyz/tpos.w,1.0);
	tpos.xy = tpos.xy/tpos.z;
	vec2 lightPos = tpos.xy*0.5+0.5;
	float truepos = sunPosition.z/abs(sunPosition.z);
	
	vec2 deltacoord = texcoord.xy-lightPos;
	vec2 grcoord = texcoord.xy + deltacoord*0.1;
	
	float grfade = pow(max(1.0-length(deltacoord*vec2(1.0,1.0/aspectRatio))*2.0,0.0),2.2);
	float centerdist = 0.25 + 0.75*pow(max(1.0-length(tpos.xy*vec2(aspectRatio,1.0)*0.5),0.0),1.5);
	float grw = 0.0;
	
	if (grfade > 0.001 && (truepos < 0.0 && sunVisibility > 0.001 || truepos > 0.0 && moonVisibility > 0.001)){
		for (int i = 0; i < 3; i++) {
		float samplea = texture2D(colortex4,getGRCoord(grcoord,deltacoord,0.25)).g;
		vec4 sampleb = texture2D(colortex1,getGRCoord(grcoord,deltacoord,0.25));
		gr += float(samplea < 0.02)*mix(vec3(1.0),vec3(sampleb.rgb)*4.0,pow(sampleb.a,0.2))*(1.0-sampleb.a)*centerdist;
		grcoord -= deltacoord*0.25;
		}
	}
	
	gr /= 3.0;
	#endif
	
/* DRAWBUFFERS:01 */
	
	gl_FragData[0] = vec4(pow(color/MAX_COLOR_RANGE,vec3(1.0/2.2)),spec);
	gl_FragData[1] = vec4(gr,0.0);

}
